/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "entityLine.h"


EntityLine::EntityLine(float interval, bool visible, Vector3 axis, 
					   Vector3 center, Vector3 rotAxis, float angle)  :
	EntityWithPosition(NULL, NULL, false, center, rotAxis, angle), 
	mInterval(interval), mAxis(axis) {
	
}
	
EntityLine::EntityLine(int size, float interval, bool visible, Material *material, Mesh *mesh, Vector3 axis, 
					   Vector3 center, Vector3 rotAxis, float angle):
	EntityWithPosition(NULL, NULL, false, center, rotAxis, angle), 
	 mInterval(interval), mAxis(axis) {
		
	for(int i=0;i<size;i++) {
		addChild(new EntityWithValueInLine (material, mesh, 0, i, interval, visible, axis), i);	
	}
}

EntityLine::~EntityLine() {
	
}

void EntityLine::insertChild(Material *material, Mesh *mesh, int value, int position) {
	addChild(new EntityWithValueInLine (material, mesh, value, position, mInterval, true, mAxis), position);
}
			
int EntityLine::getValue(int position) {
		
	if(children.find(position) != children.end()) {
		EntityWithValueInLine *entity = dynamic_cast <EntityWithValueInLine*> (children[position]);
		return entity->getValue();	
	} else {
		return -1;
	}	
}
		
bool EntityLine::setValue(int value, int position){
	if(children.find(position) != children.end()) {
		EntityWithValueInLine *entity = dynamic_cast <EntityWithValueInLine*> (children[position]);
		entity->setValue(value);
		return true;
	} else {
		return false;
	}
}

bool EntityLine::setValueAndMaterial(int value, Material *material, int position) {
	if(children.find(position) != children.end()) {
		EntityWithValueInLine *entity = dynamic_cast <EntityWithValueInLine*> (children[position]);
		entity->setValue(value);
		entity->setMaterial(material);
		return true;
	} else {
		return false;
	}
}
		
void EntityLine::setClickable(){
	mNumClick = -1;

	for(std::map<int, Entity*>::iterator it = children.begin(); it!=children.end(); it++) {
		Entity *child = (*it).second;
		int index = (*it).first;
		child->setNumClick(index);
	}
}



