/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _DISPLAY3D_H_
#define _DISPLAY3D_H_

#include <set>


#include "vector3.h"
#include "glUtil.h"
#include "camera.h"
#include "light.h"
#include "fog.h"

#include "sky.h"
#include "waterPlane.h"
#include "mirror.h"

#include "shadowMap.h"
#include "frameBufferObject.h"

#include "entity.h"

#include "shaderManager.h"
#include "./graphicOptions.h"


class Display3D
{
public:
	Display3D();
	~Display3D();
	
	void setRenderingMode (renderingMode rendering);
	void setMaxLightSources (int maxLightSources);
	renderingMode getRenderingMode ();
		
	void setCamera(Camera *camera);
	void setFog(Fog *fog);
	void setSky(Sky *sky);
	void setSkyPlane(SkyPlane *skyPlane);
	void setWaterPlane(WaterPlane *waterPlane);
		
	Camera* getCamera();	
	Fog* getFog();
	Sky* getSky();
		
	void setAmbientColors(Color groundColor, Color skyColor, Vector3 hemiAxis);
		
		
	void addEntity(Entity *entity);
	void addMirror(Mirror *mirror);
		
	void addLightPoint(LightPoint *lightP);
	void addLightDir(LightDir *lightD);
	void addLightSpot(LightSpot *lightS);

	int getLightKey();
		
	void activateLights();
	void releaseLights();

	std::set<int> getMaterialKeys();		
		
	int getMinDirLights();
	int getMaxDirLights();
	int getMinSpotLights();
	int getMaxSpotLights();
	int getMinPointLights();
	int getMaxPointLights();
		
		
	void init(textureMode texturing);
	void compileShaders();
		
	void addMaterialsInTheSet(std::set<Material*> &matSet);
	void addMeshesInTheSet(std::set<std::pair<Mesh*, int> > &meshSet);	
		
	bool isInViewport(int x, int y);				
	int getHit(int bufSize, int xMouse, int yMouse, float delX, float delY, float distMax);
	void renderClickableObjects();
		
	void renderObjects();
	void renderOccluders();
		
	void renderScene();
		
	void ambiantPass();
	void spotLightPass(int i);
	void dirLightPass(int i);
	
	void renderMirroredScene(int i);
	void renderMirrorSurface(int i);
		
	void renderObjectsReflectionInTheSea();
	void renderWater();
	void renderSky();
	void renderSkyPlane();
	void renderReflectedSky();
		
	void render(GraphicOptions *graphics);
		
protected:
	Camera *mpCamera;
	Fog *mpFog;
	ShadowMap *mpShadowMap;
	Sky* mpSky;
	SkyPlane *mpSkyPlane;	
		
	WaterPlane *mpWaterPlane;
	FrameBufferObject *mpFBO;
				
	std::vector<Entity*> entityTree;
	std::vector<Mirror*> mirrorList;
	
	Color mGroundColor;
	Color mSkyColor;
	Vector3 mHemiAxis;
	
	int mMaxLightSources;
		
	LightDirList lightsD;
	LightSpotList lightsS;		
	LightPointList lightsP;
		
	int lightKey; //nbLightsPoint + 8*nbLightsSpot + 8*8*nbLightsDir
		
	std::set<int> materialKeys;	
		
	renderingMode mRendering;		
};

#endif // _DISPLAY3D_H_
