/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "perFragLightShader.h"

PerFragLightShader::PerFragLightShader(int nbLightsDir, int nbLightsSpot, int nbLightsPoint,
									   int nbColorMaps, int nbNormalMaps, int nbGlossMaps, int nbEmiMaps) :
MaterialShader(nbLightsDir,nbLightsSpot,nbLightsPoint, nbColorMaps, nbNormalMaps, nbGlossMaps, nbEmiMaps) {

	ShaderWriter writer;
	
	//vertex shader sources
	std::vector<std::string> vsFilenames;
	std::vector<std::string> vsGeneratedSources;
	std::string vsMainSourceFile;

	writer.setNbLights(nbLightsDir, nbLightsSpot, nbLightsPoint);
	writer.setNbMaps(nbColorMaps, nbNormalMaps, nbGlossMaps, nbEmiMaps);
		
	vsGeneratedSources.push_back(writer.writeLightingVertexShaderSourceCode()); 
	vsMainSourceFile = "perFragLight_main.vert";
	
	
	//fragment shader sources
	std::vector<std::string> fsFilenames;
	std::vector<std::string> fsGeneratedSources;
	std::string fsMainSourceFile;
	
	fsFilenames.push_back("perFragLight_lightingFunctions.frag");
	
	fsGeneratedSources.push_back(writer.writeMaterialFragmentShaderSourceCode());					 
	fsGeneratedSources.push_back(writer.writeLightingFragmentShaderSourceCode()); 
	fsMainSourceFile = "perFragLight_main.frag";
			
													 
	id = loadProgram(vsFilenames, vsGeneratedSources, vsMainSourceFile, fsFilenames, fsGeneratedSources, fsMainSourceFile);
							
	if(id == 0) {
		std::cerr << "The per fragment lighting shader cannot be loaded." << std::endl;
	} else {
		setLocations ();
		
		//provisoire
		//setAmbientColors(Color(0.05, 0.9, 0.15, 1.), Color(0., 0.08, 0.04,1.), Vector3(0.,1.,0.));
		
	}	
	


}
