/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "shaderLoader.h"

ShaderLoader::ShaderLoader(GLenum type,
						   std::vector<std::string> filenames,
						   std::vector<std::string> generatedSources,
						   std::string mainSourceFile) : mType(type) {
							  
	loadSources(filenames, generatedSources, mainSourceFile);
	mNbSources =  filenames.size()+generatedSources.size()+1;	
	id = loadShader ();					   
}

ShaderLoader::ShaderLoader(GLenum type, std::vector<std::string> filename) : mType(type) {
	  
	mNbSources = filename.size();	
	mSources = new char* [mNbSources];
	
	for(int i=0;i<mNbSources;i++) {
		// load the source code
		mSources[i] = loadSource(filename[i]);
		if(mSources[i] == NULL) {
			for(int j=0;j<i;j++) {
				delete mSources[j];
			}
			delete mSources;
			mSources = NULL;
		} else {
			id = loadShader ();					   
		}	
	}
}


ShaderLoader::~ShaderLoader() {	
	for(int i=0;i<mNbSources;i++) {
		delete mSources[i];
	}
	delete mSources;
	
	glDeleteShader(id);
}


GLuint ShaderLoader::getId() {
	return id;
}



/************************/
/*	protected functions	*/
/************************/


char* ShaderLoader::loadSource(std::string filename) {
	std::string path = fileMana->getShadersDir() + "/" + filename;
	
	size_t sizePath = path.size() + 1;
    char * buffer = new char[ sizePath ];
    strncpy( buffer, path.c_str(), sizePath ); 	
	
	char *src = NULL;   // shader source code
	FILE *fp = NULL; 
	long size;          // size of the file
	long i;
    
	fp = fopen(buffer, "r");
	if(fp == NULL) {
		std::cerr << "cannot open the file "<< filename << " \n";
		return NULL;
	}
	
	fseek(fp, 0, SEEK_END);
	size = ftell(fp);

	//go to the beginning of the file
	rewind(fp);
    
    //memory allocation
	src = new char[size + 1];
	if(src == NULL) {
		fclose(fp);
		std::cerr << "cannot allocate memory! \n";
		return NULL;
	}
    
	//read the file
	for(i=0; i<size; i++)
		src[i] = fgetc(fp);
    
	//last char has to be '\0' */
	src[size] = '\0';
    
	fclose(fp);
    
	return src;		
}


void ShaderLoader::loadSources(std::vector<std::string> filenames,
								std::vector<std::string> generatedSources, 
								std::string mainSourceFile) {

	size_t size;
	int nb = filenames.size() + generatedSources.size() + 1;
									  
	mSources = new char* [nb];
	
	for(int i=0;i<filenames.size();i++) {
		mSources[i] = loadSource(filenames[i]);
	}	
	
	for(int i=0;i<generatedSources.size();i++) {
		size = generatedSources[i].size() +1;
		mSources[filenames.size()+i] = new char[size];
		strncpy( mSources[filenames.size()+i], generatedSources[i].c_str(), size ); 
	}
	mSources[nb-1] = loadSource(mainSourceFile);	
}



GLuint ShaderLoader::loadShader() {
	GLuint shader = 0;
	GLsizei logsize = 0;
	GLint compile_status = GL_TRUE;
	char *log = NULL;
	
	shader = glCreateShader(mType);
	if(shader == 0) {
		std::cerr << "Unable to create the shader." << std::endl;
		return 0;
	}	
	
	glShaderSource(shader, mNbSources, (const GLchar**) mSources, NULL);
	glCompileShader(shader);	

	// is the compilation successful ?
	glGetShaderiv(shader, GL_COMPILE_STATUS, &compile_status);
	if(compile_status != GL_TRUE) {
		// the compilation has failed :(
		glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &logsize);
        
		log = new char[logsize + 1];
		if(log == NULL) {
			std::cerr << "Cannot allocate memory!" << std::endl;
			return 0;
		}

		std::fill(log, log + logsize +1, '\0');
        
		glGetShaderInfoLog(shader, logsize, &logsize, log);
		std::cerr << "Unable to compile the shader. " << std::endl;
		std::cerr << log << std::endl;
		        
		delete log;
		glDeleteShader(shader);
        
		return 0;
	}
    
	return shader;		
}







